require 'yaml'
require 'spinna'

module Spinna
  # Takes care of reading the configuration file and exposing
  # the settings.
  class Config
    # Default number of albums to keep in the history.
    DEFAULT_HISTORY_SIZE = 50

    # Default number of albums to pick when spinna is run.
    DEFAULT_NUMBER_OF_PICKS = 10

    # The directory where spinna stores its data.
    attr_accessor :data_dir

    # The directory where all the albums are stored.
    attr_accessor :source_dir

    # The number previously picked albums to keep in the
    # history. Defaults to DEFAULT_NUMBER_OF_PICKS.
    attr_accessor :history_size

    # The number of picks to fetch if not explicitly given.
    attr_accessor :number_of_picks

    # The name of the config file to load. (ex: config.yml)
    attr_accessor :config_file

    def initialize(opts = {})
      @data_dir = opts[:data_dir] || ENV['SPINNA_DATA_DIR'] || File.join(Dir.home, '.spinna')
      @config_file = opts[:config_file] || 'config.yml'
      ensure_data_dir_exists
      ensure_configuration_exists
      parse_configuration
    end

    private

    def parse_configuration
      config = read_configuration_file
      @source_dir = config['source_dir']
      @history_size = config['history_size'] || DEFAULT_HISTORY_SIZE
      @number_of_picks = config['number_of_picks'] || DEFAULT_NUMBER_OF_PICKS
      validate_config
    end

    def validate_config
      unless source_dir
        raise Spinna::InvalidConfigFileError, "You must set a source_dir in your configuration file."
      end

      unless Dir.exists?(source_dir)
        raise Spinna::InvalidSourceDirError, "The source_dir #{source_dir} does not exist."
      end
    end

    def ensure_data_dir_exists
      if !File.exists?(data_dir)
        Dir.mkdir(data_dir, 0700)
        raise Spinna::ConfigFileNotFoundError, "Could not find the configuration file. Please create it at #{config_path}."
      end
    end

    def ensure_configuration_exists
      unless config_file_exists?
        raise Spinna::ConfigFileNotFoundError, "Could not find the configuration file. Please create it at #{config_path}."
      end
    end

    def read_configuration_file
      begin
        config = YAML.load_file(File.join(data_dir, config_file))
        raise StandardError unless config.is_a?(Hash)
      rescue StandardError
        raise Spinna::InvalidConfigFileError, "Could not parse the configuration file. Please make sure it is valid YAML."
      end
      config
    end

    def config_path
      File.join(data_dir, config_file)
    end

    def config_file_exists?
      File.exists?(config_path)
    end
  end
end
