require 'spinna/config'

module Spinna
  # Takes care of tracking the picks that have been recently made
  # so that they are not picked again too soon. The log is persisted
  # in ~/.spinna/history.log by default.
  class History
    # The configuration of the application.
    attr_reader :config

    # An array that represents the log entries.
    # The oldest entries are first.
    attr_reader :log

    def initialize(config)
      @config = config
      read_history_log
    end

    # Add album at the end of the history log unless it’s
    # already in there.
    def append(album)
      @log << album unless @log.include?(album)
      trim if full?
    end

    # Is the given album in the history already?
    def include?(album)
      @log.include?(album)
    end

    # Saves the history to file after removing old picks
    # from the log if needed.
    def save
      write_history_to_disk
    end

    # Returns the path to the history log file on the filesystem.
    def history_path
      File.join(config.data_dir, 'history.log')
    end

    private

    # Are there more albums in the log than there should be?
    def full?
      @log.size > config.history_size
    end

    # Deletes the old albums that are now outdated depending
    # on the history_size.
    def trim
      @log = @log.last(config.history_size)
    end

    def write_history_to_disk
      File.open(history_path, 'w+') do |f|
        @log.each { |album| f.puts album }
      end
    end

    # Reads the history.log file and stores all the lines
    # in an array.
    def read_history_log
      if !history_file_exists?
        @log = [] and return
      end

      @log = File.read(history_path).split("\n")
    end

    def history_file_exists?
      File.exists?(history_path)
    end
  end
end
